@@ -1,3 +1,5 @@
+import datetime
+from dateutil.tz import tzutc
 from unittest.mock import create_autospec
 
 import pytest
@@ -19,6 +21,10 @@ def spa(mock_api, mock_account):
     return spa
 
 
+async def test_spa(spa):
+    assert str(spa)
+
+
 async def test_get_status(mock_api, spa):
     mock_api.request.return_value = {
         "ambientTemperature": 65.6,
@@ -94,6 +100,7 @@ async def test_get_status(mock_api, spa):
 
     status = await spa.get_status()
     assert status.state == "NORMAL"
+    assert str(status)
 
     pf = status.primary_filtration
     assert pf.mode == status.primary_filtration.PrimaryFiltrationMode.NORMAL
@@ -297,6 +304,7 @@ async def test_get_errors(mock_api, spa):
     errors = await spa.get_errors()
     assert len(errors) == 1
     error = errors[0]
+    assert str(error)
     assert error.title == "Flow Switch Stuck Open"
 
 
@@ -318,3 +326,104 @@ async def test_get_reminders(mock_api, spa):
     reminder = reminders[0]
     assert reminder.id == "id1"
     assert reminder.name == "name1"
+
+
+async def test_get_debug_status(mock_api, spa):
+    mock_api.request.return_value = {
+        "debugStatus": {
+            "battery": {"percentCharge": 82.2, "voltage": 4.04},
+            "freeMemory": 45520,
+            "lastResetReason": "RESET_REASON_USER",
+            "powerStatus": "DC",
+            "resetCount": 133,
+            "signal": {"quality": 37, "strength": -89},
+            "uptime": {
+                "connection": 2059053,
+                "system": 2059161,
+                "tubController": 2059159,
+            },
+        }
+    }
+
+    debug_status = await spa.get_debug_status()
+    assert debug_status is not None
+
+
+async def test_get_energy_usage(mock_api, spa):
+    mock_api.request.return_value = {"buckets": []}
+    usage = await spa.get_energy_usage(
+        smarttub.Spa.EnergyUsageInterval.DAY,
+        datetime.date(2021, 1, 1),
+        datetime.date(2021, 1, 31),
+    )
+    mock_api.request.assert_called_with(
+        "POST",
+        f"spas/{spa.id}/energyUsage",
+        {"start": "2021-01-01", "end": "2021-01-31", "interval": "DAY"},
+    )
+    assert usage == []
+
+
+async def test_set_heat_mode(mock_api, spa):
+    await spa.set_heat_mode(smarttub.Spa.HeatMode.AUTO)
+    mock_api.request.assert_called_with(
+        "PATCH", f"spas/{spa.id}/config", {"heatMode": "AUTO"}
+    )
+
+
+async def test_set_temperature(mock_api, spa):
+    await spa.set_temperature(38.3)
+    mock_api.request.assert_called_with(
+        "PATCH", f"spas/{spa.id}/config", {"setTemperature": 38.3}
+    )
+
+
+async def test_toggle_clearray(mock_api, spa):
+    await spa.toggle_clearray()
+    mock_api.request.assert_called_with("POST", f"spas/{spa.id}/clearray/toggle", None)
+
+
+async def test_set_temperature_format(mock_api, spa):
+    await spa.set_temperature_format(smarttub.Spa.TemperatureFormat.FAHRENHEIT)
+    mock_api.request.assert_called_with(
+        "POST", f"spas/{spa.id}/config", {"displayTemperatureFormat": "FAHRENHEIT"}
+    )
+
+
+async def test_set_date_time(mock_api, spa):
+    with pytest.raises(ValueError):
+        await spa.set_date_time()
+
+    await spa.set_date_time(date=datetime.date(2021, 1, 1))
+    mock_api.request.assert_called_with(
+        "POST", f"spas/{spa.id}/config", {"dateTimeConfig": {"date": "2021-01-01"}}
+    )
+    await spa.set_date_time(time=datetime.time(12, 45))
+    mock_api.request.assert_called_with(
+        "POST", f"spas/{spa.id}/config", {"dateTimeConfig": {"time": "12:45"}}
+    )
+
+
+async def test_secondary_filtration_cycle(mock_api, spa):
+    cycle = smarttub.SpaSecondaryFiltrationCycle(
+        spa,
+        **{
+            "lastUpdated": "2020-07-09T19:39:52.961Z",
+            "mode": "AWAY",
+            "status": "INACTIVE",
+        },
+    )
+    assert cycle.last_updated == datetime.datetime(
+        2020, 7, 9, 19, 39, 52, 961000, tzinfo=tzutc()
+    )
+    assert (
+        cycle.mode == smarttub.SpaSecondaryFiltrationCycle.SecondaryFiltrationMode.AWAY
+    )
+    assert cycle.status == smarttub.SpaSecondaryFiltrationCycle.CycleStatus.INACTIVE
+
+    await cycle.set_mode(
+        smarttub.SpaSecondaryFiltrationCycle.SecondaryFiltrationMode.FREQUENT
+    )
+    mock_api.request.assert_called_with(
+        "PATCH", f"spas/{spa.id}/config", {"secondaryFiltrationConfig": "FREQUENT"}
+    )
